%Copyright (C) 2005  Estrade-Mercadier

%This program is free software; you can redistribute it and/or
%modify it under the terms of the GNU General Public License
%as published by the Free Software Foundation; either version 2
%of the License, or (at your option) any later version.

%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.

function res = conndef2 (dim,type)

## CONNDEF2
## 
## This function makes a connectivity matrix of dim dimension.
## You can specify the type of connectivity : 'minimal' or
## 'maximal' respectively corresponding to the 4-connectivity
## and the 8-connectivity.
## 
## M = conndef2 (dim,type)
## 
## M connectivity matrix 
## dim dimension of the connectivity matrix
## type : 'minimal' or 'maximal'
## 
## Functions necessary to use CONNDEF
## ----------------------------------
##   ones.m
##   zeros.m
##
## 

## -*- texinfo -*-
##
## @deftypefn {Function File} {@var{conn} = } conndef2 (@var{num_dims}, @var{type})
##
## Creates a connectivity array
##
## @code{conn=conndef(num_dims,type)} creates a connectivity array
## (@var{CONN}) of @var{num_dims} dimensions and which type is defined
## by @var{type} as follows:
## @table @code
## @item minimal
## Neighbours touch the central element on a (@var{num_dims}-1)-dimensional
## surface.
## @item maximal
## Neighbors touch the central element in any way. Equivalent to
## @code{ones(repmat(3,1,@var{num_dims}))}.
## @end table
##
## @end deftypefn

if dim <2
    
    sprintf('error: the dimension must be superior or equal to 2.\n');
    
else 
    if type == 'maximal'
        
    	% if the connectivity is maximal, every points in the dim-matrix will
	% be in the neighborhood.
        res = ones(3*ones(1,dim));
    
    else 
        if type == 'minimal'
    
            res = zeros(3*ones(1,dim));
            
            if dim == 2
            
                for k = 2:2:8
                    res(k)=1;
                end
                
                res(5)=1;
                
            else
        
                res((3^dim+1)/2-3^(dim-1)) = 1;
        
		% the dim-dimentional core is in fact the dim-1 neighborhood matrix
                res(3^(dim-1)+1:2*3^(dim-1)) = conndef2(dim-1,type);
        
                res((3^dim+1)/2+3^(dim-1)) = 1;
                
            end
        
        else 
        
            printf('error: you must use a type wich will be minimal or maximal.\n');
            
        end
        
    end
end
