%Copyright (C) 2005  Berge-Gladel

%This program is free software; you can redistribute it and/or
%modify it under the terms of the GNU General Public License
%as published by the Free Software Foundation; either version 2
%of the License, or (at your option) any later version.

%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.

function coefficient = dctmtx(n)

%   DCTMTX Compute discrete cosine transform matrix.
%   D = DCTMTX(N) returns the N-by-N DCT transform matrix.  D*A
%   is the DCT of the columns of A and D'*A is the inverse DCT of
%   the columns of A. So we have a matrix A N-by-N.
%
%   The two-dimensional DCT of A can be computed
%   as D*A*D'. This computation is sometimes faster than using
%   DCT2, especially if you are computing large number of small
%   DCT's, because D needs to be determined only once.
%
%   Class Support
%   -------------
%   N is a scalar of class double. D is returned as a matrix of
%   class double.

mat1=zeros(n,n);

for i=1:n
    for j=1:n
        mat1(i,j)=j-1;
        mat2(i,j)=i-1;
    end
end

coefficient = sqrt ( 2 / n) * cos ( pi * (2 * mat1 + 1) .* mat2 / (2 *n ));
coefficient(1,:) = coefficient(1,:) / sqrt(2);