%Copyright (C) 2005  Carvalho-Mariel

%This program is free software; you can redistribute it and/or
%modify it under the terms of the GNU General Public License
%as published by the Free Software Foundation; either version 2
%of the License, or (at your option) any later version.

%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.

function [IMG_R,IMG_G,IMG_B] = imclose(varargin)

% IM2 = imclose (image,SE)
% [R1,G1,B1] = imclose (r,g,b,SE)
% 
% IMCLOSE Close image.
% 
% IM2 = IMCLOSE(IM, SE) performs morphological closing on the grayscale or binary image IM with the structuring element SE. SE must be a single structuring element object, as opposed to an array of objects.
% 
% [R1, G1, B1] = IMCLOSE(R, G, B, SE) performs morphological closing on the RGB image with the structuring element SE.
% 
% Class support
% -------------
% 
% IMAGE can be a binary or a grayscale image, then IM2 is in the same class.
% R, G and B are RGB class.
% SE must be an array containing 1s and 0s. 

if(nargin != 2 && nargin != 4)
	usage("IMG = imclose(im, mask) OU [R,G,B] = imclose(r,g,b, mask)");
	
elseif(nargin == 2)
    	im = varargin{1};
    	if ~isnumeric(im) | ~isreal(im)
        	error('Image must be a real matrix.');
    	endif
    	
	SE = varargin{2};
    	if ~isnumeric(SE) | ~isreal(SE)
        	error('SE must be a real matrix.');
    	endif

	img = imerode(imdilate(im,SE),SE);

	IMG_R = img;
	IMG_G = img;
	IMG_B = img;
else
    	imr = varargin{1};
    	if ~isnumeric(imr) | ~isreal(imr)
        	error('Red image must be a real matrix.');
    	end
    	img = varargin{2};
    	if ~isnumeric(img) | ~isreal(img)
        	error('Green image must be a real matrix.');
    	end
    	imb = varargin{3};
    	if ~isnumeric(imb) | ~isreal(imb)
        	error('Blue image must be a real matrix.');
    	end
	SE = varargin{4};
    	if ~isnumeric(SE) | ~isreal(SE)
        	error('SE must be a real matrix.');
    	endif
	
	IMG_R = imerode(imdilate(imr,SE),SE);
	IMG_G = imerode(imdilate(img,SE),SE);
	IMG_B = imerode(imdilate(imb,SE),SE);
endif
endfunction
