%Copyright (C) 2005  Carvalho-Mariel

%This program is free software; you can redistribute it and/or
%modify it under the terms of the GNU General Public License
%as published by the Free Software Foundation; either version 2
%of the License, or (at your option) any later version.

%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.

function [IMG_R,IMG_G,IMG_B] = imtophat(varargin)

% IM2 = imtophat (image,SE) 	Function File
% [R1,G1,B1] = imtophat (r,g,b,SE) 	Function File
% 
% IMTOPHAT Perform top hat filtering.
% 
% IM2 = IMTOPHAT(IM, SE) performs morphological top hat filtering on the grayscale or binary image IM using the the structuring element SE. SE must be a single structuring element object, as opposed to an array of objects.
% 
% [R1, G1, B1] = imtophat(R, G, B, SE) performs morphological top hat filtering on the RGB image with the structuring element SE.
% 
% Class support
% -------------
% 
% IMAGE can be a binary or a grayscale image, then IM2 is in the same class.
% R, G and B are RGB class.
% SE must be an array containing 1s and 0s. 

if(nargin != 2 && nargin != 4)
	usage("IMG = imdilate(im, mask) OU [R,G,B] = imdilate(r,g,b, mask)");
	
elseif(nargin == 2)
    	im = varargin{1};
    	if ~isnumeric(im) | ~isreal(im)
        	error('Image must be a real matrix.');
    	endif
    	
	SE = varargin{2};
    	if ~isnumeric(SE) | ~isreal(SE)
        	error('SE must be a real matrix.');
    	endif

	img_open = imopen(im,SE);

	IMG_TOP_HAT = im & ~img_open;

	IMG_R = IMG_TOP_HAT;
	IMG_G = IMG_TOP_HAT;
	IMG_B = IMG_TOP_HAT;
else
    	imr = varargin{1};
    	if ~isnumeric(imr) | ~isreal(imr)
        	error('Red image must be a real matrix.');
    	end
    	img = varargin{2};
    	if ~isnumeric(img) | ~isreal(img)
        	error('Green image must be a real matrix.');
    	end
    	imb = varargin{3};
    	if ~isnumeric(imb) | ~isreal(imb)
        	error('Blue image must be a real matrix.');
    	end
	SE = varargin{4};
    	if ~isnumeric(SE) | ~isreal(SE)
        	error('SE must be a real matrix.');
    	endif
	
	img_r_open = imopen(imr,SE);
	img_g_open = imopen(img,SE);
	img_b_open = imopen(imb,SE);

	IMG_R = imr - img_r_open;
	IMG_G = img - img_g_open;
	IMG_B = imb - img_b_open;

	IMG_R = ~(IMG_R < 0) .* IMG_R;
	IMG_G = ~(IMG_G < 0) .* IMG_G;
	IMG_B = ~(IMG_B < 0) .* IMG_B;
endif
endfunction
